#!/usr/bin/python
import sys
import ldif
import argparse
import socket
import json
import rpm
from dmliteshell import executor
from datetime import datetime

class SystemInfo:
  def __init__(self, config=None):
    self.ts=rpm.TransactionSet()
    self.config=config
  def rpmversion(self, package):
    mi=self.ts.dbMatch("name", package)
    if mi.count()==0:
      return 1
    elif mi.count() >1:
      return 2
    else:
      entry=mi.next()
      return entry['version']
  def getprotinfo(self, prot):
    if prot == "http":
      interfaceversion="1.1"
      interfacename="http"
      implname="lcgdm-dav"
      implversion=str(self.rpmversion(implname))
      return interfacename, interfaceversion, implname, implversion
    if prot == "gsiftp":
      interfacename="gsiftp"
      interfaceversion="1.0"
      implname="dpm-dsi"
      implversion=str(self.rpmversion(implname))
      return interfacename, interfaceversion, implname, implversion
    if prot == "root":
      interfacename="root"
      interfaceversion="3.1"
      implname="dpm-xrootd"
      implversion=str(self.rpmversion(implname))
      return interfacename, interfaceversion, implname, implversion
    if prot == "srm":
      interfacename="srm"
      interfaceversion="2.2"
      implname="dpm-srm-server-mysql"
      implversion=str(self.rpmversion(implname))
      return interfacename, interfaceversion, implname, implversion
  def getsysinfo(self, component):
    if component=="dome":
      return self.rpmversion("dmlite-dome")
    if component=="dmlite":
      return self.rpmversion("dmlite-libs")
    return None
  def getspaces(self):
    """Find all the StorageShares on this DPM
    and return an array of them"""
    e=executor.DomeExecutor(self.config.cert, self.config.key, "/etc/grid-security/certificates/", "", "")
    gqt, err=e.getquotatoken("https://"+self.config.host+":1094/domehead/command","/", 0, 1)
    if err==True:
      print sys.argv[0]+": Error contacting Dome"
      sys.exit(1)
    try:
      jgqt=json.loads(gqt)
    except Exception as X:
      print sys.argv[0]+": Error loading Dome output"
      print X
      sys.exit(1)
    ret=[]
    for space in jgqt.keys():
      if isinstance(jgqt[space]["groups"], dict):
        groups = jgqt[space]["groups"].values()
      if isinstance(jgqt[space]["groups"], list):
        groups = jgqt[space]["groups"]
      groups.remove("root")
      jgqt[space]["groups"]=groups
    return jgqt

class Entry:
  """Base class for all GLUE2 entries"""
  def __init__(self):
    # An array of child objects in the ldap hierarchy
    self.Children=[]
    # This will be the attribute taken for the DN
    self.name=""
    # This will be the attribute which points to the parent
    self.foreignkey=""
    self.Attributes={}
    self.Attributes["GLUE2EntityCreationTime"]=[datetime.utcnow().replace(microsecond=0).isoformat()+'Z']
    self.ldif_writer=ldif.LDIFWriter(sys.stdout)
  def add_child(self, entry):
    self.Children.append(entry)
  def set_foreign_key(self, fkey):
    self.Attributes[self.foreignkey]=[fkey]
  def print_out(self, parent=None):
    """ Recursively print out with all children"""
    if parent == None:
      dn=self.name+"="+self.Attributes[self.name][0]
    else:
      dn=self.name+"="+self.Attributes[self.name][0] + "," + parent
    self.ldif_writer.unparse(dn, self.Attributes)
    for e in self.Children:
      e.print_out(dn)

class Service(Entry):
  """A GLUE2Service"""
  def __init__(self, config):
    Entry.__init__(self)
    self.name="GLUE2ServiceID"
    self.Attributes["GLUE2ServiceID"]=["Service_"+config.host]
    self.Attributes["GLUE2ServiceType"]=["org.dpm.storage"]
    self.Attributes["GLUE2ServiceQualityLevel"]=["production"]
    self.Attributes["GLUE2ServiceAdminDomainForeignKey"]=[config.sitename]
    self.Attributes["ObjectClass"]=["GLUE2Service", "GLUE2StorageService"]

class Endpoint(Entry):
  """A GLUE2Endpoint"""
  def __init__(self, endpointurl, interfacename, interfaceversion, implname, implversion):
    Entry.__init__(self)
    self.name="GLUE2EndpointID"
    self.foreignkey="GLUE2EndpointServiceForeignKey"
    self.Attributes["GLUE2EndpointID"]=["Endpoint_"+endpointurl]
    self.Attributes["GLUE2EndpointURL"]=[endpointurl]
    self.Attributes["GLUE2EndpointInterfaceName"]=[interfacename]
    self.Attributes["GLUE2EndpointInterfaceVersion"]=[interfaceversion]
    self.Attributes["GLUE2EndpointImplementationName"]=[implname]
    self.Attributes["GLUE2EndpointImplementationVersion"]=[implversion]
    self.Attributes["GLUE2EndpointQualityLevel"]=["production"]
    self.Attributes["GLUE2EndpointHealthState"]=["ok"]
    self.Attributes["GLUE2EndpointServingState"]=["production"]
    self.Attributes["GLUE2EndpointServiceForeignKey"]=["XXX"]
    self.Attributes["ObjectClass"]=["GLUE2StorageEndpoint", "GLUE2Endpoint"]

class Policy(Entry):
  """A GLUE2Policy"""
  def __init__(self, qtname, groups):
    Entry.__init__(self)
    self.name="GLUE2PolicyID"
    self.foreignkey="GLUE2AccessPolicyEndpointForeignKey"
    self.Attributes["GLUE2PolicyID"]=["Policy_"+qtname]
    self.Attributes["GLUE2PolicyRule"]=groups
    self.Attributes["GLUE2PolicyScheme"]=["org.glite.standard"]
    self.Attributes["GLUE2AccessPolicyEndpointForeignKey"]=["XXX"]
    self.Attributes["ObjectClass"]=["GLUE2AccessPolicy"]

class Manager(Entry):
  """A GLUE2Policy"""
  def __init__(self, domeversion, dmliteversion, hostname):
    Entry.__init__(self)
    self.name="GLUE2ManagerID"
    self.foreignkey="GLUE2ManagerServiceForeignKey"
    self.Attributes["GLUE2ManagerID"]=["Manager_"+hostname]
    self.Attributes["GLUE2ManagerProductName"]=["DPM"]
    self.Attributes["GLUE2ManagerProductVersion"]=[domeversion]
    self.Attributes["GLUE2EntityOtherInfo"]=["dmliteversion="+dmliteversion]
    self.Attributes["GLUE2ManagerServiceForeignKey"]=["XXX"]
    self.Attributes["ObjectClass"]=["GLUE2StorageManager","GLUE2Manager"]

class Share(Entry):
  """A GLUE2StorageShare"""
  def __init__(self, path, pool, groups, qtname):
    Entry.__init__(self)
    self.name="GLUE2ShareID"
    self.foreignkey="GLUE2ShareServiceForeignKey"
    self.Attributes["GLUE2ShareID"]=["Share_"+qtname]
    self.Attributes["GLUE2ShareDescription"]=groups
    self.Attributes["GLUE2StorageShareServingState"]=["production"]
    self.Attributes["GLUE2ShareServiceForeignKey"]=["XXX"]
    self.Attributes["GLUE2StorageShareSharingID"]=[pool]
    self.Attributes["GLUE2StorageShareAccessLatency"]=["1"]
    self.Attributes["GLUE2StorageSharePath"]=[path]
    self.Attributes["ObjectClass"]=["GLUE2Share", "GLUE2StorageShare"]

class ShareCapacity(Entry):
  """A GLUE2StorageShareCapacity"""
  def __init__(self, qtname, qtspace, pathused):
    Entry.__init__(self)
    self.name="GLUE2StorageShareCapacityID"
    self.foreignkey="GLUE2StorageShareCapacityStorageShareForeignKey"
    self.Attributes["GLUE2StorageShareCapacityID"]=["Capacity_"+qtname]
    self.Attributes["GLUE2StorageShareCapacityTotalSize"]=[qtspace]
    self.Attributes["GLUE2StorageShareCapacityUsedSize"]=[pathused]
    self.Attributes["GLUE2StorageShareCapacityType"]=["online"]
    self.Attributes["GLUE2StorageShareCapacityStorageShareForeignKey"]=["XXXX"]
    self.Attributes["ObjectClass"]=["GLUE2StorageShareCapacity"]

